from setuptools import setup, find_packages

setup(
    name = "z3c.image",
    version = "1.0.0",
    author = "Zope Contributors",
    author_email = "office@mopacreative.com",
    description = "Image utils for zope3",
    license = "ZPL 2.1",
    keywords = "zope3 image",
    url='http://svn.zope.org/z3c.image',
    classifiers = [
        'Development Status :: 5 - Production/Stable',
        "License :: OSI Approved :: Zope Public License",
        "Framework :: Zope3",
        ],
    packages = find_packages('src'),
    include_package_data = True,
    package_dir = {'':'src'},
    namespace_packages = ['z3c'],
    extras_require = dict(test=['zope.app.testing',
                                'zope.app.server',
                                'zope.app.authentication',
                                'zope.app.securitypolicy',
                                'zope.app.zcmlfiles',
                                'zope.lifecycleevent',
                                'zope.testbrowser',
                                'zope.testing',
                               ]),
    install_requires = ['setuptools',
    # 'PILwoTk' is a package containing a version of PIL that doesn't
    # magically sniff for the Tk installation.
                        'PILwoTk',
                        'zope.app.cache',
                        'zope.app.file',
                        'zope.cachedescriptors',
                        'zope.datetime',
                        'zope.dublincore',
                        'zope.interface',
                        'zope.publisher',
                        'zope.security',
                        ],
    zip_safe=False,
    )
